wtf.factory('DatabaseInstall', function($q, $http) {
  		var f = {};
  		
		function errorCB(err) {
			// console.log("Error processing SQL: "+err.code);
			// console.log(err);

			f.deferred.reject(err);
		}

		function createTableFailed(err) {
			// console.log("Error processing SQL: "+err.code);
			// console.log(err);

			f.deferred.reject(err);
		}

		///////////////

		function createAirlines(tx) {
			tx.executeSql('DROP TABLE IF EXISTS airline');
			tx.executeSql('CREATE TABLE airline (id INTEGER, name TEXT, alternate_name TEXT, iata_code TEXT, oaci_code TEXT, importance INTEGER)', [], createAirlineSuccess, createTableFailed);		
		}

		function createAirlineSuccess(tx) {
			var len = f.airlines.length;
			for (var i = 0; i < len; i++) {
				var item = f.airlines[i];
				tx.executeSql('INSERT INTO airline (id, name, alternate_name, iata_code, oaci_code, importance) VALUES (?, ?, ?, ?, ?, ?)', [item.id, item.name, item.alternate_name, item.iata_code, item.oaci_code, item.importance]);
				f.airlines[i] = undefined;
			}

			createAirports(tx);
		}

		function createAirports(tx) {
			tx.executeSql('DROP TABLE IF EXISTS airport');
			tx.executeSql('CREATE TABLE airport (id INTEGER, name TEXT, official_name TEXT, iata_code TEXT, oaci_code TEXT, importance INTEGER)', [], createAirportSuccess, createTableFailed);		
		}

		function createAirportSuccess(tx) {
			var len = f.airports.length;
			for (var i = 0; i < len; i++) {
				var item = f.airports[i];
				tx.executeSql('INSERT INTO airport (id, name, official_name, iata_code, oaci_code, importance) VALUES (?, ?, ?, ?, ?, ?)', [item.id, item.name, item.official_name, item.iata_code, item.oaci_code, item.importance]);
				f.airports[i] = undefined;
			}

			createAirlineReview(tx);
		}

		function createAirlineReview(tx) {
			tx.executeSql('DROP TABLE IF EXISTS airline_review');
			tx.executeSql(	'	CREATE TABLE airline_review ('+
							'		id INTEGER PRIMARY KEY AUTOINCREMENT, '+
							'		plateform_id INTERGER, '+
							'		departure_date DATE, '+
							'		title TEXT, '+
							'		text TEXT, '+
							'		airline_id INTEGER, '+
							'		departure_airport_id INTEGER, '+
							'		arrival_airport_id INTEGER, '+
							'		classe TEXT,'+
							'		rate_service INT DEFAULT -1, '+ // tinyint(2)
							'		rate_confort INT DEFAULT -1, '+
							'		rate_security INT DEFAULT -1, '+
							'		rate_global INT, '+  
							'		rate_value INT, '+
							'		rate_food INT DEFAULT -1, '+
							'		rate_luggage INT DEFAULT -1, '+
							'		rate_boarding INT DEFAULT -1, '+
							'		status INT DEFAULT 0, '+
							'		favorite_picture_id INT DEFAULT 0, '+
							'		display_order INT DEFAULT 0, '+
							'		recommand INT DEFAULT 1, '+ 
							'		use_again INT DEFAULT 1, '+
							'		creation_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP, '+
							'		update_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP'+
							'	)', [], createAirlineReviewSuccess, createTableFailed);		
		}

		function createAirlineReviewSuccess(tx) {
			tx.executeSql('CREATE INDEX airline_review_status_idx ON airline_review (status)');
			tx.executeSql('CREATE INDEX airline_review_airline_id_idx ON airline_review (airline_id)');
			tx.executeSql('CREATE INDEX airline_revie_departure_airport_id_idx ON airline_review (departure_airport_id)');
			tx.executeSql('CREATE INDEX airline_review_arrival_airport_id_idx ON airline_review (arrival_airport_id)');
			tx.executeSql('CREATE INDEX airline_review_favorite_picture_id_idx ON airline_review (favorite_picture_id)');


			// tx.executeSql('INSERT INTO airline_review (departure_date, title, text, airline_id, departure_airport_id, arrival_airport_id, classe, rate_global, rate_value, display_order) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', ['2014-10-12', 'Review 1', 'Content of review 1', '4', '2', '4', 'b', 5, 2, 5]);
			// tx.executeSql('INSERT INTO airline_review (departure_date, title, text, airline_id, departure_airport_id, arrival_airport_id, classe, rate_global, rate_value, display_order) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', ['2013-11-23', 'Review 2', 'Content of review 2', '20', '243', '423', 'b', 4, 3, 4]);
			// tx.executeSql('INSERT INTO airline_review (departure_date, title, text, airline_id, departure_airport_id, arrival_airport_id, classe, rate_global, rate_value, display_order) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', ['2012-12-30', 'Review 3', 'Content of review 3', '24', '246', '483', 'b', 1, 3, 3]);
			// tx.executeSql('INSERT INTO airline_review (departure_date, title, text, airline_id, departure_airport_id, arrival_airport_id, classe, rate_global, rate_value, display_order) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', ['2011-03-23', 'Review 4', 'Content of review 4', '26', '249', '413', 'b', 4, 3, 2]);
			// tx.executeSql('INSERT INTO airline_review (departure_date, title, text, airline_id, departure_airport_id, arrival_airport_id, classe, rate_global, rate_value, display_order) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', ['2010-02-01', 'Review 5', 'Content of review 5', '29', '240', '433', 'b', 5, 2, 1]);

			createReviewPicture(tx);
		}

		function createReviewPicture(tx) {
			tx.executeSql(	'	DROP TABLE IF EXISTS review_picture');
			tx.executeSql(	'	CREATE TABLE review_picture ('+
							'		id INTEGER PRIMARY KEY AUTOINCREMENT, '+
							'		plateform_id INTEGER, '+
							'		description TEXT, '+
							'		file TEXT, '+
							'		thumbnail TEXT, '+
							'		display_order INT DEFAULT 0, '+
							'		related_id INTEGER, '+
							'		related_table TEXT, '+
							'		status INT DEFAULT 0, '+
							'		creation_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP, '+
							'		update_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP'+
							'	)', [], createReviewPictureSuccess, createTableFailed);	
		}

		function createReviewPictureSuccess(tx) {
			tx.executeSql('CREATE INDEX review_picture_related_idx ON review_picture (related_id, related_table)');
			tx.executeSql('CREATE INDEX review_picture_status_idx ON review_picture (status)');

			// tx.executeSql('INSERT INTO review_picture (description, related_id, related_table) VALUES (?, ?, ?)', ['Lorem ipsum Lorem ipsum Lorem ipsum', 5, 'airline_review']);
			// tx.executeSql('INSERT INTO review_picture (description, related_id, related_table) VALUES (?, ?, ?)', ['Lorem ipsum Lorem ipsum Lorem ipsum', 5, 'airline_review']);
			// tx.executeSql('INSERT INTO review_picture (description, related_id, related_table) VALUES (?, ?, ?)', ['Lorem ipsum Lorem ipsum Lorem ipsum', 5, 'airline_review']);

			querySuccess(tx);
		}

		function querySuccess(tx) {
			// trace('DB installed')
			f.deferred.resolve();
		}

		function loadAirports(db) {
			$http.get('resources/database/airports.json').success(function(items) {
				f.airports = items;
				db.transaction(createAirlines, errorCB);
			});
		}

		f.run = function(db) {
			f.deferred = $q.defer();
			$http.get('resources/database/airlines.json').success(function(items) {
				f.airlines = items;
				loadAirports(db);
			});

			return f.deferred.promise;
		}
		return f;
	});	